@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2021 V.A. Sole, ESRF - D. Dale CHESS
+# Copyright (C) 2004-2022 V.A. Sole, ESRF - D. Dale CHESS
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -31,6 +31,7 @@ __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 import sys
 import os
 import posixpath
+import time
 import gc
 import re
 from operator import itemgetter
@@ -64,6 +65,7 @@ except ImportError:
                 raise
             _logger.info("Cannot open %s. Trying in SWMR mode" % filename)
             return h5py.File(filename, "r", libver='latest', swmr=True)
+from PyMca5.PyMcaIO import HDF5Utils
 
 from PyMca5.PyMcaGui import PyMcaQt as qt
 safe_str = qt.safe_str
@@ -159,19 +161,18 @@ def h5py_sorting(object_list, sorting_list=None):
         _logger.warning("Probably all entries do not have the key %s" % sorting_key)
         return object_list
 
+
 def _get_number_list(txt):
     rexpr = '[/a-zA-Z:_-]'
     nbs= [float(w) for w in re.split(rexpr, txt) if w not in ['',' ']]
     return nbs
 
+
 class BrokenLink(object):
     pass
 
+
 class QRLock(qt.QMutex):
-
-    """
-    """
-
     def __init__(self):
         qt.QMutex.__init__(self, qt.QMutex.Recursive)
 
@@ -227,6 +228,7 @@ class RootItem(object):
         del self._children[idx]
         del self._identifiers[idx]
 
+
 class H5NodeProxy(object):
     def sortChildren(self, column, order):
         #print("sort children called with ", column, order)
@@ -237,6 +239,22 @@ class H5NodeProxy(object):
             self.__sorting_list = None
         self.__sorting_order = order
 
+    def raw_keys(self):
+        for node in self.children:
+            yield node.name
+
+    def raw_values(self):
+        node = self.getNode(self.name)
+        if hasattr(node, "values"):
+            return node.values()
+        else:
+            # spech5 does not implement values() prior to silx 0.6.0
+            for _, value in self.raw_items():
+                yield value
+
+    def raw_items(self):
+        return self.getNode(self.name).items()
+
     @property
     def children(self):
         if not self.hasChildren:
@@ -246,12 +264,11 @@ class H5NodeProxy(object):
             # obtaining the lock here is necessary, otherwise application can
             # freeze if navigating tree while data is processing
             if 1: #with self.file.plock:
-                items = list(self.getNode(self.name).items())
-                if posixpath.dirname(self.name) == "/":
-                    # top level item
-                    doit = True
-                else:
-                    doit = False
+                try:
+                    items = list(self.raw_items())
+                except Exception:
+                    items = []
+                    _logger.warning("Cannot obtain items list. Ignoring")
                 try:
                     # better handling of external links
                     finalList = h5py_sorting(items, sorting_list=self.__sorting_list)
@@ -266,17 +283,13 @@ class H5NodeProxy(object):
                                                                finalList[i][0])
                     self._children = [H5NodeProxy(self.file, i[1], self)
                                       for i in finalList]
-                except:
-                    #one cannot afford any error, so I revert to the old
+                except Exception:
+                    # one cannot afford any error, so I revert to the old
                     # method where values where used instead of items
                     if 1 or _logger.getEffectiveLevel() == logging.DEBUG:
                         raise
-                    else:
-                        # tmpList = list(self.getNode(self.name).values())
-                        # spech5 does not implement values() prior to silx 0.6.0
-                        tmpList = list(map(self.getNode(self.name).__getitem__,
-                                           self.getNode(self.name).keys()))
-                        finalList = tmpList
+                    tmpList = list(self.raw_values())
+                    finalList = tmpList
                     for i in range(len(finalList)):
                         finalList[i]._posixPath = posixpath.join(self.name,
                                                                items[i][0])
@@ -436,6 +449,22 @@ class H5FileProxy(H5NodeProxy):
         else:
             return H5NodeProxy(self.file, self.file[path], self)
 
+    def raw_keys(self):
+        if isinstance(self.file, h5py.File):
+            file_path = self.file.filename
+            data_path = self.name
+            return HDF5Utils.safe_hdf5_group_keys(file_path, data_path=data_path)
+        else:
+            return super().raw_keys()
+
+    def raw_values(self):
+        for _, value in self.raw_items():
+            yield value
+
+    def raw_items(self):
+        for data_path in self.raw_keys():
+            yield data_path, self.getNode(data_path)
+
 
 class FileModel(qt.QAbstractItemModel):
     """
@@ -465,66 +494,73 @@ class FileModel(qt.QAbstractItemModel):
         return 4
 
     def data(self, index, role):
-        if role == qt.Qt.DisplayRole:
-            item = self.getProxyFromIndex(index)
-            column = index.column()
-            if column == 0:
-                if isinstance(item, H5FileProxy):
-                    return MyQVariant(os.path.basename(item.file.filename))
-                else:
-                    if hasattr(item, "name"):
-                        return MyQVariant(posixpath.basename(item.name))
+        try:
+            if role == qt.Qt.DisplayRole:
+                item = self.getProxyFromIndex(index)
+                column = index.column()
+                if column == 0:
+                    if isinstance(item, H5FileProxy):
+                        try:
+                            return MyQVariant(os.path.basename(item.file.filename))
+                        except:
+                            _logger.critical("Cannot retrieve file name")
+                            return MyQVariant("Unknown file. Please refresh")
                     else:
-                        # this can only happen with the root
-                        return MyQVariant("/")
-            if column == 1:
-                showtitle = True
-                if showtitle:
-                    if hasattr(item, 'type'):
-                        if item.type in ["Entry", "NXentry"]:
-                            if hasattr(item, "children"):
-                                children = item.children
-                                names = [posixpath.basename(o.name) for o in children]
-                                if "title" in names:
-                                    idx = names.index("title")
-                                    node = children[idx].getNode()
-                                    if hasattr(node, "shape") and len(node.shape):
-                                        #stored as an array of strings???
-                                        #return just the first item
-                                        if hasattr(node, "asstr"):
+                        if hasattr(item, "name"):
+                            return MyQVariant(posixpath.basename(item.name))
+                        else:
+                            # this can only happen with the root
+                            return MyQVariant("/")
+                if column == 1:
+                    showtitle = True
+                    if showtitle:
+                        if hasattr(item, 'type'):
+                            if item.type in ["Entry", "NXentry"]:
+                                if hasattr(item, "children"):
+                                    children = item.children
+                                    names = [posixpath.basename(o.name) for o in children]
+                                    if "title" in names:
+                                        idx = names.index("title")
+                                        node = children[idx].getNode()
+                                        if hasattr(node, "shape") and len(node.shape):
+                                            #stored as an array of strings???
+                                            #return just the first item
+                                            if hasattr(node, "asstr"):
+                                                try:
+                                                    return MyQVariant("%s" % node.asstr()[()][0])
+                                                except:
+                                                    return MyQVariant("%s" % node[()][0])
+                                        else:
+                                            #stored as a string
                                             try:
-                                                return MyQVariant("%s" % node.asstr()[()][0])
+                                                try:
+                                                    return MyQVariant("%s" % node.asstr()[()])
+                                                except:
+                                                    return MyQVariant("%s" % node[()])
                                             except:
-                                                return MyQVariant("%s" % node[()][0])
-                                    else:
-                                        #stored as a string
-                                        try:
-                                            try:
-                                                return MyQVariant("%s" % node.asstr()[()])
-                                            except:
-                                                return MyQVariant("%s" % node[()])
-                                        except:
-                                            # issue #745
-                                            return MyQVariant("Unknown %s" % node)
-                            else:
-                                _logger.critical("Entry %s has no children" % item.name)
-                return MyQVariant(item.type)
-            if column == 2:
-                return MyQVariant(item.shape)
-            if column == 3:
-                return MyQVariant(item.dtype)
-        elif role == qt.Qt.ForegroundRole:
-            item = self.getProxyFromIndex(index)
-            column = index.column()
-            if column == 0:
+                                                # issue #745
+                                                return MyQVariant("Unknown %s" % node)
+                                else:
+                                    _logger.critical("Entry %s has no children" % item.name)
+                    return MyQVariant(item.type)
+                if column == 2:
+                    return MyQVariant(item.shape)
+                if column == 3:
+                    return MyQVariant(item.dtype)
+            elif role == qt.Qt.ForegroundRole:
+                item = self.getProxyFromIndex(index)
+                column = index.column()
+                if column == 0:
+                    if hasattr(item, "color"):
+                        return MyQVariant(qt.QColor(item.color))
+            elif role == qt.Qt.ToolTipRole:
+                item = self.getProxyFromIndex(index)
                 if hasattr(item, "color"):
-                    return MyQVariant(qt.QColor(item.color))
-        elif role == qt.Qt.ToolTipRole:
-            item = self.getProxyFromIndex(index)
-            if hasattr(item, "color"):
-                if item.color == qt.Qt.blue:
-                    return MyQVariant("Item has a double click NXdata associated action")
-        return MyQVariant()
+                    if item.color == qt.Qt.blue:
+                        return MyQVariant("Item has a double click NXdata associated action")
+            return MyQVariant()
+        except:
+            return MyQVariant("Unhandled exception filling tree. Reload?")
 
     def getNodeFromIndex(self, index):
         try:
@@ -827,7 +863,7 @@ class HDF5Widget(FileView):
         ddict['dtype'] = item.dtype
         ddict['shape'] = item.shape
         ddict['color'] = item.color
-        ddict['mouse'] = self._lastMouse * 1
+        ddict['mouse'] = getattr(self, '_lastMouse', 'left') * 1
         self.sigHDF5WidgetSignal.emit(ddict)
 
     def getSelectedEntries(self):
@@ -949,12 +985,15 @@ class Hdf5SelectionDialog(qt.QDialog):
         self.selectedItemUri = self._lastEvent['file'] + "::" + self._lastEvent['name']
         self.accept()
 
-    def exec_(self):
+    def exec(self):
         with h5open(self.filename) as hdf5File:
             self.fileModel.appendPhynxFile(hdf5File, weakreference=True)
-            ret = qt.QDialog.exec_(self)
+            ret = qt.QDialog.exec(self)
         return ret
 
+    def exec_(self):
+        return self.exec()
+
 
 def _getFilenameDialog(parent=None):
     """Open a dialog to select a file in a filesystem tree view.
@@ -987,7 +1026,7 @@ def getDatasetValueDialog(filename=None, message=None, parent=None):
     """
     hdf5Dialog = Hdf5SelectionDialog(parent, filename, message,
                                      "dataset")
-    ret = hdf5Dialog.exec_()
+    ret = hdf5Dialog.exec()
     if not ret:
         return None
 
@@ -1002,7 +1041,7 @@ def getDatasetValueDialog(filename=None, message=None, parent=None):
 def getDatasetUri(parent=None, filename=None, message=None):
     # TODO: Accept a filter for type of dataset
     hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "dataset")
-    ret = hdf5Dialog.exec_()
+    ret = hdf5Dialog.exec()
     if not ret:
         return None
     selectedHdf5Uri = hdf5Dialog.selectedItemUri
@@ -1011,7 +1050,7 @@ def getDatasetUri(parent=None, filename=None, message=None):
 def getGroupUri(parent=None, filename=None, message=None):
     # TODO: Accept a filter for a particular attribute (NXclass)
     hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "dataset")
-    ret = hdf5Dialog.exec_()
+    ret = hdf5Dialog.exec()
     if not ret:
         return None
     selectedHdf5Uri = hdf5Dialog.selectedItemUri
@@ -1019,7 +1058,7 @@ def getGroupUri(parent=None, filename=None, message=None):
 
 def getUri(parent=None, filename=None, message=None):
     hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "any")
-    ret = hdf5Dialog.exec_()
+    ret = hdf5Dialog.exec()
     if not ret:
         return None
     selectedHdf5Uri = hdf5Dialog.selectedItemUri
@@ -1033,7 +1072,7 @@ def getDatasetDialog(filename=None, value=False, message=None, parent=None):
                                      parent=parent)
 
     hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "dataset")
-    ret = hdf5Dialog.exec_()
+    ret = hdf5Dialog.exec()
     if not ret:
         return None
     selectedHdf5Uri = hdf5Dialog.selectedItemUri
@@ -1051,7 +1090,7 @@ def getGroupNameDialog(filename=None, message=None, parent=None):
     :return: HDF5 group name
     """
     hdf5Dialog = Hdf5SelectionDialog(parent, filename, message, "group")
-    ret = hdf5Dialog.exec_()
+    ret = hdf5Dialog.exec()
     if not ret:
         return None
 
@@ -1075,6 +1114,6 @@ if __name__ == "__main__":
             print(phynxFile[ddict['name']].dtype, phynxFile[ddict['name']].shape)
     fileView.sigHDF5WidgetSignal.connect(mySlot)
     fileView.show()
-    ret = app.exec_()
+    ret = app.exec()
     app = None
     sys.exit(ret)
